//
// Created by divin on 23-7-13.
//

#include "IRInstruction.h"
#include "ValueRef.h"
#include "codegen/AsmBuilder.hpp"

using namespace std;

int IRInstruction::vRegCount = 0;

IRInstruction::IRInstruction(InstType_Enum instType): Instruction(instType) {
    setVRegisterCount(0);
}

DummyInstruction::DummyInstruction() : IRInstruction(DUMMY){}

AddGlobalInstruction::AddGlobalInstruction(ValueRef *dst, Type *varType, ValueRef *initVal) : IRInstruction(AddGlb){
    this->dst = dst;
    this->varType = varType;
    this->initVal = initVal;

    //TODO:use-def集
}

void AddGlobalInstruction::codegen(AsmBuilder *pBuilder) const {
    auto &ss = pBuilder->getUnit()->global_vars;
//    cout << this->varType->type << endl;
//TODO:other types
//    static_cast<Symbol>(this->dst->get_Ref())
   string name = ((Symbol*)this->dst)->name;
//    pBuilder->getUnit()->global_symbol.insert(name);

    if(Type_Enum(this->varType->type) == INT32TYPE){
            if(this->initVal!= nullptr) {
                ss << ".data" << '\n';
                ss << this->dst->get_Ref().substr(1) << ":\n";
                ss << "\t.word\t" << this->initVal->get_Ref() << '\n';
            } else {
                ss << ".bss" << '\n';
                ss << this->dst->get_Ref().substr(1) << ":\n";
                ss << "\t.skip\t4" << '\n';
            }
    }
}

AllocaInstruction::AllocaInstruction(ValueRef* dst, Type* varType) : IRInstruction(ALLOCA){
    this->dst = dst;
    this->varType = varType;

    addDef(dst);
}

LoadInstruction::LoadInstruction(ValueRef *dst, ValueRef *src) : IRInstruction(LOAD){
    this->dst = dst;
    this->src = src;

    addDef(dst);
    addUse(src);
}

void LoadInstruction::codegen(AsmBuilder *pBuilder, std::map<std::string, int> &offset_table,
                              std::map<std::string, int> &size_table, int frameSize) {
//    cout << this->src->type << endl;
    if(this->src->get_Ref()[0] == '@'){
    // if (pBuilder->mUnit->global_symbol.find(this->src->get_Ref()) != pBuilder->mUnit->global_symbol.end()) {
        auto la = new LoadImmInstruction(pBuilder->getBlock(),
                                                LoadImmInstruction::LA,
                                                new MachineOperand(MachineOperand::VREG, getVRegister()),
                                                new MachineOperand(MachineOperand::LABEL, this->src->get_Ref().substr(1)));
        pBuilder->getBlock()->insertInst(la);
        auto lw = new LoadMInstruction(pBuilder->getBlock(),
                                       LoadMInstruction::LW,
                                       new MachineOperand(MachineOperand::VREG, this->dst->get_Ref()),
                                       new MachineOperand(MachineOperand::IMM, 0),
                                       new MachineOperand(MachineOperand::VREG, getLastVRegister()));
        pBuilder->getBlock()->insertInst(lw);
    } else{
        if(RefType(this->src->type) == SYMBOL){
//        cout << 1 << endl;
//        load var from stack
            auto offsetIter = offset_table.find(this->src->get_Ref());
            auto lw1 = new LoadMInstruction(pBuilder->getBlock(),
                                            LoadMInstruction::LW,
                                            new MachineOperand(MachineOperand::VREG, this->dst->get_Ref()),
                                            new MachineOperand(MachineOperand::IMM, -20-offsetIter->second),
                                            new MachineOperand(MachineOperand::REG, IREGISTER::FP));
            pBuilder->getBlock()->insertInst(lw1);
        }
    }
}

StoreInstruction::StoreInstruction(ValueRef* dst,ValueRef* src) : IRInstruction(STORE){
    this->dst = dst;
    this->src = src;

    addDef(dst);
    addUse(src);
}

void StoreInstruction::codegen(AsmBuilder *pBuilder, std::map<std::string, int> &offset_table,
                               std::map<std::string, int> &size_table, int frameSize) {
//        li      a5,1
//        sw      a5,-20(s0)
//        stringtoint
//    cout << this->dst->get_Ref() << endl;
    if(this->dst->get_Ref()[0] == '@'){
//        cout << "storeGlobal" << endl;
        if(RefType(this->src->type) == IntConst){
//        cout << "intConst" << endl;
//            auto liImm = new LoadImmInstruction(pBuilder->getBlock(),
//                                                LoadImmInstruction::LI,
//                                                new MachineOperand(MachineOperand::VREG, getVRegister()),
//                                                new MachineOperand(MachineOperand::IMM, strtol(this->src->get_Ref().c_str(), nullptr, 10)));
//            pBuilder->getBlock()->insertInst(liImm);
            loadImmIntoReg(pBuilder,strtol(this->src->get_Ref().c_str(), nullptr, 10));
//            high
            auto liAddress = new LoadImmInstruction(pBuilder->getBlock(),
                                                LoadImmInstruction::LUI,
                                                new MachineOperand(MachineOperand::VREG, getVRegister()),
                                                new MachineOperand(MachineOperand::LABEL, this->dst->get_Ref().substr(1)));
            pBuilder->getBlock()->insertInst(liAddress);
//              low
            auto addAddress = new BinaryMInstruction(pBuilder->getBlock(),
                                                     BinaryMInstruction::ADDI,
                                                    new MachineOperand(MachineOperand::VREG, getLastVRegister()),
                                                    new MachineOperand(MachineOperand::VREG, getLastVRegister()),
                                                    new MachineOperand(MachineOperand::LABEL, "%lo("+this->dst->get_Ref().substr(1)+")"));
            pBuilder->getBlock()->insertInst(addAddress);
//            store
            auto store = new StoreMInstruction(pBuilder->getBlock(),
                                              StoreMInstruction::SW,
                                              new MachineOperand(MachineOperand::VREG, getLastVRegister(2)),
                                              new MachineOperand(MachineOperand::IMM, 0),
                                              new MachineOperand(MachineOperand::VREG, getLastVRegister()));
            pBuilder->getBlock()->insertInst(store);
        }
    } else{
        if(RefType(this->src->type) == IntConst){
//        cout << "intConst" << endl;
            auto liImm = new LoadImmInstruction(pBuilder->getBlock(),
                                                LoadImmInstruction::LI,
                                                new MachineOperand(MachineOperand::VREG, getVRegister()),
                                                new MachineOperand(MachineOperand::IMM, strtol(this->src->get_Ref().c_str(), nullptr, 10)));
            pBuilder->getBlock()->insertInst(liImm);
            auto offsetIter = offset_table.find(this->dst->get_Ref());
            auto sw = new StoreMInstruction(pBuilder->getBlock(),
                                            StoreMInstruction::SW,
                                            new MachineOperand(MachineOperand::VREG, getLastVRegister()),
                                            new MachineOperand(MachineOperand::IMM, -20-offsetIter->second),
                                            new MachineOperand(MachineOperand::REG, IREGISTER::FP));
            pBuilder->getBlock()->insertInst(sw);
        }else if(RefType(this->src->type) == IntVar){
//        cout << "intVar" << endl;
            auto offsetIter = offset_table.find(this->dst->get_Ref());
            auto sw = new StoreMInstruction(pBuilder->getBlock(),
                                            StoreMInstruction::SW,
                                            new MachineOperand(MachineOperand::VREG, this->src->get_Ref()),
                                            new MachineOperand(MachineOperand::IMM, -20-offsetIter->second),
                                            new MachineOperand(MachineOperand::REG, IREGISTER::FP));
            pBuilder->getBlock()->insertInst(sw);
        }
    }
}

BinaryInstruction::BinaryInstruction(ValueRef* dst,ValueRef* src1, ValueRef* src2, binaryType opTy) : IRInstruction(BINARY){
    this->dst = dst;
    this->src1 = src1;
    this->src2 = src2;
    this->opTy = opTy;

    addDef(dst);
    addUse(src1);
    addUse(src2);
}


void BinaryInstruction::outPut() {
    string tb[] = {" add i32 "," sub i32 "," mul i32 "," sdiv i32 "," srem i32 "," fadd float ", " fsub float ", " fmul float ", " fdiv float " ," not exist"};
    int idx = opTy;
    if(this->dst->type == FloatVar) idx += 5;
    cout << dst->get_Ref() + " =" + tb[idx] + src1->get_Ref() + ", " + src2->get_Ref() << endl;
}

void BinaryInstruction::codegen(AsmBuilder *pBuilder, std::map<std::string, int> &offset_table,
                                std::map<std::string, int> &size_table, int frameSize) {
    if(this->opTy == ADD){
//        cout << "add" << endl;
        if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntVar){
            auto addi = new BinaryMInstruction(pBuilder->getBlock(),
                                               BinaryMInstruction::ADDW,
                                               new MachineOperand(MachineOperand::VREG, this->dst->get_Ref()),
                                               new MachineOperand(MachineOperand::VREG, this->src1->get_Ref()),
                                               new MachineOperand(MachineOperand::VREG, this->src2->get_Ref()));
            pBuilder->getBlock()->insertInst(addi);
        }else if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntConst){
            Addiw(pBuilder, BinaryMInstruction::ADDIW, this->dst->get_Ref(), this->src1->get_Ref(), strtol(this->src2->get_Ref().c_str(), nullptr, 10));
        }else if(RefType(this->src1->type) == IntConst && RefType(this->src2->type) == IntVar){
            Addiw(pBuilder, BinaryMInstruction::ADDIW, this->dst->get_Ref(), this->src2->get_Ref(), strtol(this->src1->get_Ref().c_str(), nullptr, 10));
        }else{
            throw NotImplemented();
        }
    }
    else if(this->opTy == SUB){
//        cout << "sub" << endl;
        if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntVar){
            BinaryInstr(pBuilder,BinaryMInstruction::SUBW,this->dst->get_Ref(), this->src1->get_Ref(),this->src2->get_Ref());
        }else if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntConst){
            Addiw(pBuilder, BinaryMInstruction::ADDIW, this->dst->get_Ref(), this->src1->get_Ref(), -strtol(this->src2->get_Ref().c_str(), nullptr, 10));
        }else if(RefType(this->src1->type) == IntConst && RefType(this->src2->type) == IntVar){
            loadImmIntoReg(pBuilder, strtol(this->src1->get_Ref().c_str(), nullptr, 10));
            BinaryInstr(pBuilder,BinaryMInstruction::SUBW,this->dst->get_Ref(), getLastVRegister(),this->src2->get_Ref());
        }else{
            throw NotImplemented();
        }
    }
    else if(this->opTy == MOD){
//        cout << "mod" << endl;
        if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntVar){
            BinaryInstr(pBuilder, BinaryMInstruction::REMW, this->dst->get_Ref(), this->src1->get_Ref(), this->src2->get_Ref());
        }else if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntConst){
            loadImmIntoReg(pBuilder,strtol(this->src2->get_Ref().c_str(), nullptr, 10));
            BinaryInstr(pBuilder, BinaryMInstruction::REMW, this->dst->get_Ref(), this->src1->get_Ref(), getLastVRegister());
        }else if(RefType(this->src1->type) == IntConst && RefType(this->src2->type) == IntVar){
            loadImmIntoReg(pBuilder,strtol(this->src1->get_Ref().c_str(), nullptr, 10));
            BinaryInstr(pBuilder, BinaryMInstruction::REMW, this->dst->get_Ref(), getLastVRegister(),this->src2->get_Ref());
        }else{
            throw NotImplemented();
        }
    }
    else if(this->opTy == DIV) {
        if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntVar){
            BinaryInstr(pBuilder, BinaryMInstruction::DIVW, this->dst->get_Ref(), this->src1->get_Ref(), this->src2->get_Ref());
        }else if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntConst){
            loadImmIntoReg(pBuilder, strtol(this->src2->get_Ref().c_str(), nullptr, 10));
            BinaryInstr(pBuilder, BinaryMInstruction::DIVW, this->dst->get_Ref(), this->src1->get_Ref(), getLastVRegister());
        }else if(RefType(this->src1->type) == IntConst && RefType(this->src2->type) == IntVar){
            loadImmIntoReg(pBuilder, strtol(this->src1->get_Ref().c_str(), nullptr, 10));
            BinaryInstr(pBuilder, BinaryMInstruction::DIVW, this->dst->get_Ref(), getLastVRegister(),this->src2->get_Ref());
        }
    }
    else if(this->opTy == MUL) {
        if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntVar){
            BinaryInstr(pBuilder, BinaryMInstruction::MULW, this->dst->get_Ref(), this->src1->get_Ref(), this->src2->get_Ref());
        }else if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntConst){
            loadImmIntoReg(pBuilder, strtol(this->src2->get_Ref().c_str(), nullptr, 10));
            BinaryInstr(pBuilder, BinaryMInstruction::MULW, this->dst->get_Ref(), this->src1->get_Ref(), getLastVRegister());
        }else if(RefType(this->src1->type) == IntConst && RefType(this->src2->type) == IntVar){
            loadImmIntoReg(pBuilder, strtol(this->src1->get_Ref().c_str(), nullptr, 10));
            BinaryInstr(pBuilder, BinaryMInstruction::MULW, this->dst->get_Ref(), getLastVRegister(),this->src2->get_Ref());
        }
    }
}

CmpInstruction::CmpInstruction(ValueRef* result, ValueRef* src1, ValueRef* src2, cmpType opTy) : IRInstruction(CMP){
    this->result = result;
    this->src1 = src1;
    this->src2 = src2;
    this->opTy = opTy;

    addDef(result);
    addUse(src1);
    addUse(src2);
}

void CmpInstruction::outPut() {
    //EQ, NE, LT, LE, GT, GE
    string tb[] = {"icmp eq i32 ","icmp ne i32 ","icmp slt i32 ","icmp sle i32 ","icmp sgt i32 ","icmp sge i32 ",
                   "fcmp oeq float ", "fcmp une float ", "fcmp olt float ", "fcmp ole float " ,"fcmp ogt float ", "fcmp oge float "};
    int idx = opTy;
    if(this->src1->type == FloatVar) idx += 6;
    cout << result->get_Ref() + " = " + tb[idx] + src1->get_Ref() + ", " + src2->get_Ref() << endl;
}

void
CmpInstruction::codegen(AsmBuilder *pBuilder, map<std::string, int> &offset_table, map<std::string, int> &size_table,
                        int frameSize) {
//    var and var
    if(cmpType(this->opTy) == NE){
        if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntVar){
            auto xor1 = new BinaryMInstruction(pBuilder->getBlock(),
                                               BinaryMInstruction::XOR,
                                               new MachineOperand(MachineOperand::VREG, getVRegister()),
                                               new MachineOperand(MachineOperand::VREG, this->src1->get_Ref()),
                                               new MachineOperand(MachineOperand::VREG, this->src2->get_Ref()));
            pBuilder->getBlock()->insertInst(xor1);
            auto sltu = new CmpMInstruction(pBuilder->getBlock(),
                                            CmpMInstruction::SLTU,
                                            new MachineOperand(MachineOperand::VREG, this->result->get_Ref()),
                                            new MachineOperand(MachineOperand::REG, ZERO),
                                            new MachineOperand(MachineOperand::VREG, getLastVRegister()));
            pBuilder->getBlock()->insertInst(sltu);
//            TODO: other types
        }else if(RefType(this->src1->type) == IntConst && RefType(this->src2->type) == IntVar){
            auto xor1 = new BinaryMInstruction(pBuilder->getBlock(),
                                               BinaryMInstruction::XORI,
                                               new MachineOperand(MachineOperand::VREG, getVRegister()),
                                               new MachineOperand(MachineOperand::IMM, strtol(this->src2->get_Ref().c_str(), nullptr,10)),
                                               new MachineOperand(MachineOperand::VREG, this->src1->get_Ref()));
            pBuilder->getBlock()->insertInst(xor1);
            auto sltu = new CmpMInstruction(pBuilder->getBlock(),
                                            CmpMInstruction::SLTU,
                                            new MachineOperand(MachineOperand::VREG, this->result->get_Ref()),
                                            new MachineOperand(MachineOperand::REG, ZERO),
                                            new MachineOperand(MachineOperand::VREG, getLastVRegister()));
            pBuilder->getBlock()->insertInst(sltu);
        }else if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntConst){
            auto xor1 = new BinaryMInstruction(pBuilder->getBlock(),
                                               BinaryMInstruction::XORI,
                                               new MachineOperand(MachineOperand::VREG, getVRegister()),
                                               new MachineOperand(MachineOperand::VREG, this->src1->get_Ref()),
                                               new MachineOperand(MachineOperand::IMM, strtol(this->src2->get_Ref().c_str(), nullptr,10)));
            pBuilder->getBlock()->insertInst(xor1);
            auto sltu = new CmpMInstruction(pBuilder->getBlock(),
                                            CmpMInstruction::SLTU,
                                            new MachineOperand(MachineOperand::VREG, this->result->get_Ref()),
                                            new MachineOperand(MachineOperand::REG, ZERO),
                                            new MachineOperand(MachineOperand::VREG, getLastVRegister()));
            pBuilder->getBlock()->insertInst(sltu);
        }
    }
    else if(cmpType(this->opTy) == EQ){
        if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntVar){
            auto xor1 = new BinaryMInstruction(pBuilder->getBlock(),
                                               BinaryMInstruction::XOR,
                                               new MachineOperand(MachineOperand::VREG, getVRegister()),
                                               new MachineOperand(MachineOperand::VREG, this->src1->get_Ref()),
                                               new MachineOperand(MachineOperand::VREG, this->src2->get_Ref()));
            pBuilder->getBlock()->insertInst(xor1);
            auto sltu = new CmpMInstruction(pBuilder->getBlock(),
                                            CmpMInstruction::SLTU,
                                            new MachineOperand(MachineOperand::VREG, this->result->get_Ref()),
                                            new MachineOperand(MachineOperand::REG, ZERO),
                                            new MachineOperand(MachineOperand::VREG, getLastVRegister()));
            pBuilder->getBlock()->insertInst(sltu);
        }else if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntConst){
            auto xor1 = new BinaryMInstruction(pBuilder->getBlock(),
                                               BinaryMInstruction::XORI,
                                               new MachineOperand(MachineOperand::VREG, getVRegister()),
                                               new MachineOperand(MachineOperand::VREG, this->src1->get_Ref()),
                                               new MachineOperand(MachineOperand::IMM, strtol(this->src2->get_Ref().c_str(), nullptr,10)));
            pBuilder->getBlock()->insertInst(xor1);
            auto sltu = new CmpMInstruction(pBuilder->getBlock(),
                                            CmpMInstruction::SLTU,
                                            new MachineOperand(MachineOperand::VREG, this->result->get_Ref()),
                                            new MachineOperand(MachineOperand::REG, ZERO),
                                            new MachineOperand(MachineOperand::VREG, getLastVRegister()));
            pBuilder->getBlock()->insertInst(sltu);
        }else if(RefType(this->src1->type) == IntConst && RefType(this->src2->type) == IntVar){
            auto xor1 = new BinaryMInstruction(pBuilder->getBlock(),
                                               BinaryMInstruction::XORI,
                                               new MachineOperand(MachineOperand::VREG, getVRegister()),
                                               new MachineOperand(MachineOperand::IMM, strtol(this->src2->get_Ref().c_str(), nullptr,10)),
                                               new MachineOperand(MachineOperand::VREG, this->src1->get_Ref()));
            pBuilder->getBlock()->insertInst(xor1);
            auto sltu = new CmpMInstruction(pBuilder->getBlock(),
                                            CmpMInstruction::SLTU,
                                            new MachineOperand(MachineOperand::VREG, this->result->get_Ref()),
                                            new MachineOperand(MachineOperand::REG, ZERO),
                                            new MachineOperand(MachineOperand::VREG, getLastVRegister()));
            pBuilder->getBlock()->insertInst(sltu);
        }
        xoriReg(pBuilder,new MachineOperand(MachineOperand::VREG, this->result->get_Ref()));
    }
    else if(cmpType(this->opTy) == LT || cmpType(this->opTy) == GE){
        if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntVar){
            auto slt = new CmpMInstruction(pBuilder->getBlock(),
                                            CmpMInstruction::SLT,
                                            new MachineOperand(MachineOperand::VREG, this->result->get_Ref()),
                                            new MachineOperand(MachineOperand::VREG, this->src1->get_Ref()),
                                            new MachineOperand(MachineOperand::VREG, this->src2->get_Ref()));
            pBuilder->getBlock()->insertInst(slt);
        }else if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntConst){
            loadImmIntoReg(pBuilder,strtol(this->src2->get_Ref().c_str(), nullptr, 10));
            auto slt = new CmpMInstruction(pBuilder->getBlock(),
                                           CmpMInstruction::SLT,
                                           new MachineOperand(MachineOperand::VREG, this->result->get_Ref()),
                                           new MachineOperand(MachineOperand::VREG, this->src1->get_Ref()),
                                           new MachineOperand(MachineOperand::VREG, getLastVRegister()));
            pBuilder->getBlock()->insertInst(slt);
        }else if(RefType(this->src1->type) == IntConst && RefType(this->src2->type) == IntVar){
            loadImmIntoReg(pBuilder,strtol(this->src1->get_Ref().c_str(), nullptr, 10));
            auto slt = new CmpMInstruction(pBuilder->getBlock(),
                                           CmpMInstruction::SLT,
                                           new MachineOperand(MachineOperand::VREG, this->result->get_Ref()),
                                           new MachineOperand(MachineOperand::VREG, getLastVRegister()),
                                           new MachineOperand(MachineOperand::VREG, this->src2->get_Ref()));
            pBuilder->getBlock()->insertInst(slt);
        }
        if(cmpType(this->opTy) == GE)
            xoriReg(pBuilder, new MachineOperand(MachineOperand::VREG, this->result->get_Ref()));
    }
    else if(cmpType(this->opTy) == LE || cmpType(this->opTy) == GT){
        if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntVar){
            auto slt = new CmpMInstruction(pBuilder->getBlock(),
                                           CmpMInstruction::SLT,
                                           new MachineOperand(MachineOperand::VREG, this->result->get_Ref()),
                                           new MachineOperand(MachineOperand::VREG, this->src2->get_Ref()),
                                           new MachineOperand(MachineOperand::VREG, this->src1->get_Ref()));
            pBuilder->getBlock()->insertInst(slt);
        }else if(RefType(this->src1->type) == IntVar && RefType(this->src2->type) == IntConst){
            loadImmIntoReg(pBuilder, strtol(this->src2->get_Ref().c_str(), nullptr, 10));
            auto slt = new CmpMInstruction(pBuilder->getBlock(),
                                           CmpMInstruction::SLT,
                                           new MachineOperand(MachineOperand::VREG, this->result->get_Ref()),
                                           new MachineOperand(MachineOperand::VREG, getLastVRegister()),
                                           new MachineOperand(MachineOperand::VREG, this->src1->get_Ref()));
            pBuilder->getBlock()->insertInst(slt);
        }else if(RefType(this->src1->type) == IntConst && RefType(this->src2->type) == IntVar){
            loadImmIntoReg(pBuilder, strtol(this->src1->get_Ref().c_str(), nullptr, 10));
            auto slt = new CmpMInstruction(pBuilder->getBlock(),
                                           CmpMInstruction::SLT,
                                           new MachineOperand(MachineOperand::VREG, this->result->get_Ref()),
                                           new MachineOperand(MachineOperand::VREG, this->src2->get_Ref()),
                                           new MachineOperand(MachineOperand::VREG, getLastVRegister()));
            pBuilder->getBlock()->insertInst(slt);
        }
        if(cmpType(this->opTy) == LE)
            xoriReg(pBuilder, new MachineOperand(MachineOperand::VREG, this->result->get_Ref()));
    }
}

BrInstruction::BrInstruction(BasicBlock* label) : IRInstruction(BR){
    this->label = label;
}

void BrInstruction::codegen(AsmBuilder *pBuilder, std::map<std::string, int> &offset_table,
                            std::map<std::string, int> &size_table, int frameSize) {
    auto br = new JumpMInstruction(pBuilder->getBlock(),
                                   JumpMInstruction::J,
                                   new MachineOperand(MachineOperand::LABEL, this->label->label));
    pBuilder->getBlock()->insertInst(br);
}

CondBrInstruction::CondBrInstruction(ValueRef* condition, BasicBlock* trueLabel, BasicBlock* falseLabel) : IRInstruction(CONDBR){
    this->condition = condition;
    this->trueLabel = trueLabel;
    this->falseLabel = falseLabel;

    addUse(condition);
}

void CondBrInstruction::codegen(AsmBuilder *pBuilder, map<std::string, int> &offset_table, map<std::string, int> &size_table,
                           int frameSize) {
    if(this->condition->get_Ref() == "1"){
        JumpLabel(pBuilder, this->trueLabel->label);
    }
    else if(this->condition->get_Ref() == "0"){
        JumpLabel(pBuilder, this->falseLabel->label);
    }
    else{
        auto bne = new BranchMInstruction(pBuilder->getBlock(),
                                          BranchMInstruction::BNE,
                                          new MachineOperand(MachineOperand::VREG,this->condition->get_Ref()),
                                          new MachineOperand(MachineOperand::REG,ZERO),
                                          new MachineOperand(MachineOperand::LABEL,this->trueLabel->label));
        pBuilder->getBlock()->insertInst(bne);
        JumpLabel(pBuilder, this->falseLabel->label);
    }
}

RetInstruction::RetInstruction(ValueRef* retVal) : IRInstruction(RET){
    this->retVal = retVal;
    if(retVal != nullptr){
        addUse(retVal);
    }
}
void RetInstruction::codegen(AsmBuilder *pBuilder, std::map<std::string, int> &offset_table,
                             std::map<std::string, int> &size_table, int frameSize) {
    if(this->retVal == nullptr){

    }else{
        if(RefType(this->retVal->type) == IntConst){
//        cout << "retIntConst" << endl;
            auto addImmToA0 = new BinaryMInstruction(pBuilder->getBlock(),
                                                     BinaryMInstruction::ADDI,
                                                     new MachineOperand(MachineOperand::REG, A0),
                                                     new MachineOperand(MachineOperand::REG, ZERO),
                                                     new MachineOperand(MachineOperand::IMM,strtol(this->retVal->get_Ref().c_str(), nullptr, 10)));
            pBuilder->getBlock()->insertInst(addImmToA0);
        }else if(RefType(this->retVal->type) == IntVar){
//        cout << "retIntVar" << endl;
            auto addIntToA0 = new BinaryMInstruction(pBuilder->getBlock(),
                                                     BinaryMInstruction::ADDI,
                                                     new MachineOperand(MachineOperand::REG, A0),
                                                     new MachineOperand(MachineOperand::VREG, this->retVal->get_Ref()),
                                                     new MachineOperand(MachineOperand::IMM, 0));
            pBuilder->getBlock()->insertInst(addIntToA0);
        }
    }
    retFunc(pBuilder, frameSize-16);
}

CallInstruction::CallInstruction(ValueRef* retVal, Function* function, vector<ValueRef*>& args) : IRInstruction(CALL){
    this->retVal = retVal;
    this->function = function;
    this->args = args;

    addDef(retVal);
    for (auto arg : args)
        addUse(arg);
}

void CallInstruction::codegen(AsmBuilder *pBuilder, std::map<std::string, int> &offset_table,
                              std::map<std::string, int> &size_table, int frameSize) {
    if(this->args.empty()){
        auto jal = new JalMInstruction(pBuilder->getBlock(),
                                       JalMInstruction::JAL,
                                       new MachineOperand(MachineOperand::REG, RA),
                                       new MachineOperand(MachineOperand::LABEL, this->function->name));
        pBuilder->getBlock()->insertInst(jal);
    } else{
        if(this->args.size()<=8){
            for(int i=0; i<this->args.size(); i++){
                cout << this->args[i]->type << endl;
                if(RefType(this->args[i]->type) == IntVar){
                    auto addArgument = new BinaryMInstruction(pBuilder->getBlock(),
                                                              BinaryMInstruction::ADDI,
                                                              new MachineOperand(MachineOperand::REG, IREGISTER::A0+i),//x2(sp)
                                                              new MachineOperand(MachineOperand::VREG, "%" + to_string(i)),
                                                              new MachineOperand(MachineOperand::IMM, 0));
                    pBuilder->getBlock()->insertInst(addArgument);
                }
            }
            auto jal = new JalMInstruction(pBuilder->getBlock(),
                                           JalMInstruction::JAL,
                                           new MachineOperand(MachineOperand::REG, RA),
                                           new MachineOperand(MachineOperand::LABEL, this->function->name));
            pBuilder->getBlock()->insertInst(jal);
        }
    }
//    return val
    if(Type_Enum(this->function->funcType->retType->type) == INT32TYPE){
        auto addi = new BinaryMInstruction(pBuilder->getBlock(),
                                           BinaryMInstruction::ADDI,
                                           new MachineOperand(MachineOperand::VREG, this->retVal->get_Ref()),
                                           new MachineOperand(MachineOperand::REG, A0),
                                           new MachineOperand(MachineOperand::IMM, 0));
        pBuilder->getBlock()->insertInst(addi);
    }
}

ZExtInstruction::ZExtInstruction(ValueRef* dst,ValueRef* src) : IRInstruction(ZEXT){
    this->dst = dst;
    this->src = src;

    addDef(dst);
    addUse(src);
}

void
ZExtInstruction::codegen(AsmBuilder *pBuilder, map<std::string, int> &offset_table, map<std::string, int> &size_table,
                         int frameSize) {
    auto and1 = new BinaryMInstruction(pBuilder->getBlock(),
                                       BinaryMInstruction::ANDI,
                                       new MachineOperand(MachineOperand::VREG, this->dst->get_Ref()),
                                       new MachineOperand(MachineOperand::VREG, this->src->get_Ref()),
                                       new MachineOperand(MachineOperand::IMM, 1));
    pBuilder->getBlock()->insertInst(and1);
}

XorInstruction::XorInstruction(ValueRef* dst, ValueRef* src) : IRInstruction(XOR){
    this->dst = dst;
    this->src = src;

    addDef(dst);
    addUse(src);
}

GEPInstruction::GEPInstruction(ValueRef* dst, ValueRef* src, ValueRef* index) : IRInstruction(GEP){
    this->dst = dst;
    this->src = src;
    this->index = index;

    addDef(dst);
    addUse(src);
    addUse(index);
}

PhiInstruction::PhiInstruction(ValueRef* symbol,ValueRef* result,int cnt) : IRInstruction(PHI){
    this->symbol = symbol;
    this->result = result;
    this->branch_cnt = cnt;

    addDef(result);
}

void PhiInstruction::outPut() {
    // %3 = phi i32 [ 0, %0 ], [ %10, %6 ]
    cout << result->get_Ref() + " = phi ";
    cout << ((result->type == IntVar)?"i32 " : "float ");
    bool not_first = false;
    for(auto[block,ref] : this->mp){
        if(not_first){
            cout << ", ";
        }
        cout << "[ " << ref->get_Ref() << ", %"<< block->label << " ]";
        not_first = true;
    }
    cout << endl;
}

FPSIInstruction::FPSIInstruction(ValueRef* dst, ValueRef* src) : IRInstruction(FPSI){
    this->dst = dst;
    this->src = src;

    addDef(dst);
    addUse(src);
}

FNegInstruction::FNegInstruction(ValueRef *dst, ValueRef *src) : IRInstruction(FNEG){
    this->dst = dst;
    this->src = src;

    addDef(dst);
    addUse(src);
}
//BitCastInstruction::BitCastInstruction(BasicBlock *block) : IRInstruction(BITCAST){}
//
//ShlInstruction::ShlInstruction(BasicBlock *block) : IRInstruction(SHL){}
//
//AShrInstruction::AShrInstruction(BasicBlock *block) : IRInstruction(ASHR){}

